from typing import Dict, List, Any
import string
import numpy as np

from blueness import module

from bluer_sandbox import NAME
from bluer_sandbox.logger import logger


NAME = module.name(__file__, NAME)


def is_anagram(word_1, word_2) -> bool:
    return sorted(list(word_1)) == sorted(list(word_2))


def normalize_and_group_and_analyze(words: List[str]):
    logger.info("processing the words: {}".format(", ".join(words)))
    words = normalize_input(words)
    logger.info("normalized to: {}".format(", ".join(words)))

    groups = group_anagrams(words)
    logger.info(f"groups {groups}")

    stats = analyze_groups(groups)

    print_report(groups, stats)


def normalize_input(words: List[str]) -> List[str]:
    # Trims, lowercases, and filters out non-alphabetic words.
    # Returns a clean list of valid words.
    return [
        word
        for word in [word.strip().lower() for word in words]
        if all(character in string.ascii_lowercase for character in word) and word
    ]


def group_anagrams(words: List[str]) -> Dict[str, List[str]]:
    # Groups normalized words into anagram clusters.
    # Uses sorted letters as the key.
    groups: Dict[str, List[str]] = {}

    for word in words:
        found: bool = False
        for keyword, group in groups.items():
            if is_anagram(word, keyword):
                group.append(word)
                found = True
                break

        if not found:
            keyword = "".join(list(word))
            groups[keyword] = [word]

    return groups


def analyze_groups(groups: Dict[str, List[str]]) -> Dict[str, Any]:
    # Returns statistics:
    # total_groups
    # largest_group
    # average_group_size
    stats: Dict[str, Any] = {
        "total_groups": len(groups),
    }

    largest_group_size = max(len(group) for group in groups.values())
    largest_group_keyword = [
        keyword for keyword, group in groups.items() if len(group) == largest_group_size
    ]
    stats["largest_group"] = largest_group_keyword[0] if largest_group_keyword else ""

    stats["average_group_size"] = float(
        np.mean(np.array([len(group) for group in groups.values()]))
    )

    return stats


def group_as_str(group: List[str]) -> str:
    return "[{}]".format(", ".join(group))


def print_report(
    groups: Dict[str, List[str]],
    stats: Dict[str, Any],
):
    # Outputs a nicely formatted report, with:
    # Total groups
    # Average group size
    # Largest group
    # All anagram clusters sorted by the first element in each

    logger.info("Total groups: {}".format(stats["total_groups"]))

    logger.info("Average group size: {:.2f}".format(stats["average_group_size"]))

    largest_group_keyword = stats["largest_group"]
    logger.info(
        "Largest group (size {}): {}".format(
            len(groups[largest_group_keyword]),
            group_as_str(groups[largest_group_keyword]),
        )
    )

    logger.info("Anagram groups:")
    for group in groups.values():
        logger.info(group_as_str(group))


def count_anagrams_in_list(
    dictionary: List[str],
    query: List[str],
) -> List[int]:
    return [count_anagrams(dictionary, query_string) for query_string in query]


def count_anagrams(
    dictionary: List[str],
    query_string: str,
) -> int:
    logger.info(f"dictionary: {dictionary}")
    logger.info(f"query_string: {query_string}")

    count = 0
    for word in dictionary:
        if is_anagram(word, query_string):
            count += 1

    return count



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from bluer_sandbox import NAME
from bluer_sandbox.village.analysis import analyze
from bluer_sandbox import env
from bluer_sandbox.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="analyze",
)
parser.add_argument(
    "--verbose",
    type=bool,
    default=0,
    help="0|1",
)
parser.add_argument(
    "--object_name",
    type=str,
    default=env.BLUER_VILLAGE_TEST_OBJECT,
)
args = parser.parse_args()

success = False
if args.task == "analyze":
    success = analyze(
        object_name=args.object_name,
        verbose=args.verbose == 1,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from bluer_sandbox import NAME
from bluer_sandbox.parser.parsing import parse
from bluer_sandbox.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="parse",
)
parser.add_argument(
    "--object_name",
    type=str,
)
parser.add_argument(
    "--filename",
    type=str,
    default="",
)
parser.add_argument(
    "--url",
    type=str,
)
args = parser.parse_args()

success = False
if args.task == "parse":
    success, _ = parse(
        url=args.url,
        object_name=args.object_name,
        filename=args.filename,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from typing import Tuple, List
import urllib.request
from urllib.parse import urljoin
from bs4 import BeautifulSoup
import re

from blueness import module
from bluer_objects import path, file
from bluer_objects import objects

from bluer_sandbox import NAME
from bluer_sandbox.parser.hashing import hash_of
from bluer_sandbox.logger import logger


NAME = module.name(__file__, NAME)


def parse(
    url: str,
    object_name: str = "",
    filename: str = "",
) -> Tuple[bool, List[str]]:
    logger.info("{}.parse({})".format(NAME, url))

    success = False
    list_of_urls: List[str] = []

    try:
        response = urllib.request.urlopen(url)
        content = response.read().decode("utf-8")
        soup = BeautifulSoup(content, "html.parser")

        list_of_urls = list({link.get("href") for link in soup.find_all("a")})
        list_of_urls = [
            url_
            for url_ in list_of_urls
            if isinstance(url_, str) and not url_.startswith("#")
        ]
        list_of_urls = [
            urljoin(url, url_) if url_.startswith("/") else url_
            for url_ in list_of_urls
        ]
        logger.info(f"found {len(list_of_urls)} url(s):")
        for index, url_ in enumerate(list_of_urls):
            logger.info(f"#{index+1: 3}: {url_}")

        if object_name:
            if not filename:
                filename = "cache/{}.html".format(hash_of(url))

            filename = objects.path_of(
                object_name=object_name,
                filename=filename,
            )

            if not path.create(file.path(filename)):
                return False, []

            with open(filename, "w") as f:
                f.write(content)

            logger.info(f"-> {filename}")

        success = True
    except Exception as e:
        logger.error(e)

    return success, list_of_urls



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_speedtest(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("dryrun,~install,~ping", mono=mono)

    return show_usage(
        [
            "@speedtest",
            f"[{options}]",
        ],
        "speedtest.",
        mono=mono,
    )



NAME = "bluer_sandbox"

ICON = "🌀"

DESCRIPTION = f"{ICON} A sandbox for ideas and experiments."

VERSION = "5.516.1"

REPO_NAME = "bluer-sandbox"

MARQUEE = "https://github.com/kamangir/assets/raw/main/blue-plugin/marquee.png?raw=true"

ALIAS = "@sandbox"


def fullname() -> str:
    return f"{NAME}-{VERSION}"



from typing import List

from bluer_sandbox.village.person import Person


class Family:
    def __init__(
        self,
        parents: List[Person] = [],
        children: List[Person] = [],
        end: int = -1,
    ):
        self.parents = parents
        self.children = children
        self.end = end

    def as_str(
        self,
        verbose: bool = False,
    ) -> str:
        return "[{}]{}{}".format(
            " + ".join(
                [person.as_str() if verbose else person.name for person in self.parents]
            ),
            (
                ""
                if not self.children
                else " -> {}".format(
                    " + ".join(
                        [
                            child.as_str() if verbose else child.name
                            for child in self.children
                        ]
                    )
                )
            ),
            ("" if self.end == -1 else f" (ended {self.end})"),
        )



import pygame
import time
import os

from bluer_objects.env import abcli_path_git

no_time_to_die = [
    ("C5", 800),
    ("B4", 800),
    ("A4", 1200),
    (None, 300),
    ("C5", 800),
    ("B4", 800),
    ("A4", 1200),
    (None, 400),
    ("G4", 600),
    ("A4", 600),
    ("A4", 800),
    ("G4", 800),
    ("F4", 800),
    ("E4", 1200),
    ("E4", 800),
    ("F4", 800),
    ("G4", 1000),
    ("F4", 1000),
    ("C5", 800),
    ("B4", 800),
    ("A4", 1200),
    ("A4", 600),
    ("G4", 600),
    ("F4", 800),
    ("G4", 800),
    ("E4", 800),
    ("C5", 1200),
    ("B4", 1000),
    ("A4", 1000),
    (None, 500),
]


# Init pygame mixer
pygame.init()
pygame.mixer.init()

# Load all note sounds from the folder
NOTE_FOLDER = os.path.join(
    abcli_path_git,
    "piano-mp3/piano-mp3",
)
note_sounds = {}

for filename in os.listdir(NOTE_FOLDER):
    if filename.endswith(".mp3"):
        note_name = filename[:-4]  # Strip .mp3
        note_sounds[note_name] = pygame.mixer.Sound(os.path.join(NOTE_FOLDER, filename))


# Playback function
def play_melody(melody):
    for note, duration in melody:
        if note is None:
            time.sleep(duration / 1000)
        else:
            sound = note_sounds.get(note)
            if sound:
                sound.play()
            time.sleep(duration / 1000)
            sound.stop()


# Play Billie Eilish - No Time To Die (simplified)
play_melody(no_time_to_die)
